@@ -16,20 +16,7 @@
    new application), to use the old API define it to 21 (this is the
    default), to use the even older 1.X API define it to 11. */
 
-#ifndef FUSE_USE_VERSION
-#define FUSE_USE_VERSION 21
-#endif
-
-/** Major version of FUSE library interface */
-#define FUSE_MAJOR_VERSION 2
-
-/** Minor version of FUSE library interface */
-#define FUSE_MINOR_VERSION 3
-
-/* This interface uses 64 bit off_t */
-#if _FILE_OFFSET_BITS != 64
-#error Please add -D_FILE_OFFSET_BITS=64 to your compile flags!
-#endif
+#include "fuse_common.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -47,6 +34,9 @@ extern "C" {
 /** Handle for a FUSE filesystem */
 struct fuse;
 
+/** Structure containing a raw command */
+struct fuse_cmd;
+
 /** Function to add an entry in a readdir() operation
  *
  * @param buf the buffer passed to the readdir() operation
@@ -56,27 +46,13 @@ struct fuse;
  * @return 1 if buffer is full, zero otherwise
  */
 typedef int (*fuse_fill_dir_t) (void *buf, const char *name,
-                                const struct stat *stat, off_t off);
+                                const struct stat *stbuf, off_t off);
 
 /* Used by deprecated getdir() method */
 typedef struct fuse_dirhandle *fuse_dirh_t;
 typedef int (*fuse_dirfil_t) (fuse_dirh_t h, const char *name, int type,
                               ino_t ino);
 
-/** Information about open files */
-struct fuse_file_info {
-    /** Open flags.  Available in open() and release() */
-    int flags;
-
-    /** File handle.  May be filled in by filesystem in open().
-        Available in all other file operations */
-    unsigned long fh;
-
-    /** In case of a write operation indicates if this was caused by a
-        writepage */
-    int writepage;
-};
-
 /**
  * The file system operations:
  *
@@ -86,10 +62,9 @@ struct fuse_file_info {
  * negated error value (-errno) directly.
  *
  * All methods are optional, but some are essential for a useful
- * filesystem (e.g. getattr).  Flush, release, fsync, opendir,
- * releasedir, fsyncdir, init and destroy are special purpose
- * methods, without which a full featured filesystem can still be
- * implemented.
+ * filesystem (e.g. getattr).  Open, flush, release, fsync, opendir,
+ * releasedir, fsyncdir, init and destroy are special purpose methods,
+ * without which a full featured filesystem can still be implemented.
  */
 struct fuse_operations {
     /** Get file attributes.
@@ -152,11 +127,13 @@ struct fuse_operations {
 
     /** File open operation
      *
-     * No creation, or trunctation flags (O_CREAT, O_EXCL, O_TRUNC)
+     * No creation, or truncation flags (O_CREAT, O_EXCL, O_TRUNC)
      * will be passed to open().  Open should check if the operation
      * is permitted for the given flags.  Optionally open may also
-     * return an arbitary filehandle in the fuse_file_info structure,
+     * return an arbitrary filehandle in the fuse_file_info structure,
      * which will be passed to all file operations.
+     *
+     * Changed in version 2.2
      */
     int (*open) (const char *, struct fuse_file_info *);
 
@@ -168,6 +145,8 @@ struct fuse_operations {
      * 'direct_io' mount option is specified, in which case the return
      * value of the read system call will reflect the return value of
      * this operation.
+     *
+     * Changed in version 2.2
      */
     int (*read) (const char *, char *, size_t, off_t, struct fuse_file_info *);
 
@@ -176,6 +155,8 @@ struct fuse_operations {
      * Write should return exactly the number of bytes requested
      * except on error.  An exception to this is when the 'direct_io'
      * mount option is specified (see read operation).
+     *
+     * Changed in version 2.2
      */
     int (*write) (const char *, const char *, size_t, off_t,
                   struct fuse_file_info *);
@@ -203,6 +184,11 @@ struct fuse_operations {
      * not possible to determine if a flush is final, so each flush
      * should be treated equally.  Multiple write-flush sequences are
      * relatively rare, so this shouldn't be a problem.
+     *
+     * Filesystems shouldn't assume that flush will always be called
+     * after some writes, or that if will be called at all.
+     *
+     * Changed in version 2.2
      */
     int (*flush) (const char *, struct fuse_file_info *);
 
@@ -217,6 +203,8 @@ struct fuse_operations {
      * have a file opened more than once, in which case only the last
      * release will mean, that no more reads/writes will happen on the
      * file.  The return value of release is ignored.
+     *
+     * Changed in version 2.2
      */
     int (*release) (const char *, struct fuse_file_info *);
 
@@ -224,6 +212,8 @@ struct fuse_operations {
      *
      * If the datasync parameter is non-zero, then only the user data
      * should be flushed, not the meta data.
+     *
+     * Changed in version 2.2
      */
     int (*fsync) (const char *, int, struct fuse_file_info *);
 
@@ -239,10 +229,12 @@ struct fuse_operations {
     /** Remove extended attributes */
     int (*removexattr) (const char *, const char *);
 
-    /** Open direcory
+    /** Open directory
      *
      * This method should check if the open operation is permitted for
      * this  directory
+     *
+     * Introduced in version 2.3
      */
     int (*opendir) (const char *, struct fuse_file_info *);
 
@@ -264,17 +256,24 @@ struct fuse_operations {
      * passes non-zero offset to the filler function.  When the buffer
      * is full (or an error happens) the filler function will return
      * '1'.
+     *
+     * Introduced in version 2.3
      */
     int (*readdir) (const char *, void *, fuse_fill_dir_t, off_t,
                     struct fuse_file_info *);
 
-    /** Release directory */
+    /** Release directory
+     *
+     * Introduced in version 2.3
+     */
     int (*releasedir) (const char *, struct fuse_file_info *);
 
     /** Synchronize directory contents
      *
      * If the datasync parameter is non-zero, then only the user data
      * should be flushed, not the meta data
+     *
+     * Introduced in version 2.3
      */
     int (*fsyncdir) (const char *, int, struct fuse_file_info *);
 
@@ -284,6 +283,8 @@ struct fuse_operations {
      * The return value will passed in the private_data field of
      * fuse_context to all file operations and as a parameter to the
      * destroy() method.
+     *
+     * Introduced in version 2.3
      */
     void *(*init) (void);
 
@@ -291,6 +292,8 @@ struct fuse_operations {
      * Clean up filesystem
      *
      * Called on filesystem exit.
+     *
+     * Introduced in version 2.3
      */
     void (*destroy) (void *);
 };
@@ -349,25 +352,6 @@ int fuse_main(int argc, char *argv[], const struct fuse_operations *op);
  * More detailed API                                           *
  * ----------------------------------------------------------- */
 
-/*
- * Create a FUSE mountpoint
- *
- * Returns a control file descriptor suitable for passing to
- * fuse_new()
- *
- * @param mountpoint the mount point path
- * @param opts a comma separated list of mount options.  Can be NULL.
- * @return the control file descriptor on success, -1 on failure
- */
-int fuse_mount(const char *mountpoint, const char *opts);
-
-/*
- * Umount a FUSE mountpoint
- *
- * @param mountpoint the mount point path
- */
-void fuse_unmount(const char *mountpoint);
-
 /**
  * Create a new FUSE filesystem.
  *
@@ -392,11 +376,11 @@ void fuse_destroy(struct fuse *f);
 /**
  * FUSE event loop.
  *
- * Requests from the kernel are processed, and the apropriate
+ * Requests from the kernel are processed, and the appropriate
  * operations are called.
  *
  * @param f the FUSE handle
- * @return 0 if no error occured, -1 otherwise
+ * @return 0 if no error occurred, -1 otherwise
  */
 int fuse_loop(struct fuse *f);
 
@@ -410,7 +394,7 @@ void fuse_exit(struct fuse *f);
 /**
  * FUSE event loop with multiple threads
  *
- * Requests from the kernel are processed, and the apropriate
+ * Requests from the kernel are processed, and the appropriate
  * operations are called.  Request are processed in parallel by
  * distributing them between multiple threads.
  *
@@ -418,7 +402,7 @@ void fuse_exit(struct fuse *f);
  * the application.
  *
  * @param f the FUSE handle
- * @return 0 if no error occured, -1 otherwise
+ * @return 0 if no error occurred, -1 otherwise
  */
 int fuse_loop_mt(struct fuse *f);
 
@@ -434,12 +418,9 @@ int fuse_loop_mt(struct fuse *f);
 struct fuse_context *fuse_get_context(void);
 
 /**
- * Invalidate cached data of a file.
+ * Obsolete, doesn't do anything
  *
- * Useful if the 'kernel_cache' mount option is given, since in that
- * case the cache is not invalidated on file open.
- *
- * @return 0 on success or -errno on failure
+ * @return -EINVAL
  */
 int fuse_invalidate(struct fuse *f, const char *path);
 
@@ -464,9 +445,6 @@ int fuse_main_real(int argc, char *argv[], const struct fuse_operations *op,
  * Advanced API for event handling, don't worry about this...  *
  * ----------------------------------------------------------- */
 
-/** Structure containing a raw command */
-struct fuse_cmd;
-
 /** Function type used to process commands */
 typedef void (*fuse_processor_t)(struct fuse *, struct fuse_cmd *, void *);
 
@@ -490,7 +468,7 @@ int fuse_loop_mt_proc(struct fuse *f, fuse_processor_t proc, void *data);
 
 /** Return the exited flag, which indicates if fuse_exit() has been
     called */
-int fuse_exited(struct fuse* f);
+int fuse_exited(struct fuse *f);
 
 /** Set function which can be used to get the current context */
 void fuse_set_getcontext_func(struct fuse_context *(*func)(void));
