@@ -16,7 +16,7 @@
 #include <linux/mm.h>
 #endif
 #include <linux/sched.h>
-#ifdef KERNEL_2_6_8_PLUS
+#ifdef KERNEL_2_6
 #include <linux/namei.h>
 #endif
 
@@ -112,6 +112,8 @@ static int fuse_lookup_iget(struct inode *dir, struct dentry *entry,
 	fuse_lookup_init(req, dir, entry, &outarg);
 	request_send(fc, req);
 	err = req->out.h.error;
+	if (!err && (!outarg.nodeid || outarg.nodeid == FUSE_ROOT_ID))
+		err = -EIO;
 	if (!err) {
 		inode = fuse_iget(dir->i_sb, outarg.nodeid, outarg.generation,
 				  &outarg.attr);
@@ -168,6 +170,10 @@ static int create_new_entry(struct fuse_conn *fc, struct fuse_req *req,
 		fuse_put_request(fc, req);
 		return err;
 	}
+	if (!outarg.nodeid || outarg.nodeid == FUSE_ROOT_ID) {
+		fuse_put_request(fc, req);
+		return -EIO;
+	}
 	inode = fuse_iget(dir->i_sb, outarg.nodeid, outarg.generation,
 			  &outarg.attr);
 	if (!inode) {
@@ -415,7 +421,7 @@ int fuse_do_getattr(struct inode *inode)
 	fuse_put_request(fc, req);
 	if (!err) {
 		if ((inode->i_mode ^ arg.attr.mode) & S_IFMT) {
-#ifndef FUSE_MAINLINE
+#ifndef KERNEL_2_6_12_PLUS
 			if (get_node_id(inode) != FUSE_ROOT_ID)
 				make_bad_inode(inode);
 #else
@@ -581,6 +587,7 @@ static int fuse_readdir(struct file *file, void *dstbuf, filldir_t filldir)
 				    filldir);
 
 	__free_page(page);
+	fuse_invalidate_attr(inode); /* atime changed */
 	return err;
 }
 
@@ -614,6 +621,7 @@ static char *read_link(struct dentry *dentry)
 		link[req->out.args[0].size] = '\0';
  out:
 	fuse_put_request(fc, req);
+	fuse_invalidate_attr(inode); /* atime changed */
 	return link;
 }
 
@@ -623,7 +631,18 @@ static void free_link(char *link)
 		free_page((unsigned long) link);
 }
 
-#ifdef KERNEL_2_6_8_PLUS
+#ifdef KERNEL_2_6_13_PLUS
+static void *fuse_follow_link(struct dentry *dentry, struct nameidata *nd)
+{
+	nd_set_link(nd, read_link(dentry));
+	return NULL;
+}
+
+static void fuse_put_link(struct dentry *dentry, struct nameidata *nd, void *c)
+{
+	free_link(nd_get_link(nd));
+}
+#elif defined(KERNEL_2_6_8_PLUS)
 static int fuse_follow_link(struct dentry *dentry, struct nameidata *nd)
 {
 	nd_set_link(nd, read_link(dentry));
@@ -756,7 +775,7 @@ static int fuse_setattr(struct dentry *entry, struct iattr *attr)
 	fuse_put_request(fc, req);
 	if (!err) {
 		if ((inode->i_mode ^ outarg.attr.mode) & S_IFMT) {
-#ifndef FUSE_MAINLINE
+#ifndef KERNEL_2_6_12_PLUS
 			if (get_node_id(inode) != FUSE_ROOT_ID)
 				make_bad_inode(inode);
 #else
@@ -807,6 +826,7 @@ static struct dentry *fuse_lookup(struct inode *dir, struct dentry *entry,
 			iput(inode);
 			return ERR_PTR(-EIO);
 		}
+		dput(alias);
 	}
 	return d_splice_alias(inode, entry);
 }
