@@ -23,10 +23,12 @@
 #include <string.h>
 #include <ctype.h>
 #include <unistd.h>
+#include <getopt.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <pwd.h>
 #include <mntent.h>
+#include <dirent.h>
 #include <sys/param.h>
 #include <sys/wait.h>
 #include <sys/stat.h>
@@ -43,15 +45,13 @@
 #define FUSE_DEV_NEW "/dev/fuse"
 #define FUSE_VERSION_FILE_OLD "/proc/fs/fuse/version"
 #define FUSE_CONF "/etc/fuse.conf"
-#define FUSE_MAJOR 10
-#define FUSE_MINOR 229
 
 static const char *progname;
 
 static int user_allow_other = 0;
 static int mount_max = 1000;
 
-static const char *get_user_name()
+static const char *get_user_name(void)
 {
     struct passwd *pw = getpwuid(getuid());
     if (pw != NULL && pw->pw_name != NULL)
@@ -65,17 +65,20 @@ static const char *get_user_name()
 static uid_t oldfsuid;
 static gid_t oldfsgid;
 
-static int drop_privs(void)
+static void drop_privs(void)
 {
-    oldfsuid = setfsuid(getuid());
-    oldfsgid = setfsgid(getgid());
-    return 0;
+    if (getuid() != 0) {
+        oldfsuid = setfsuid(getuid());
+        oldfsgid = setfsgid(getgid());
+    }
 }
 
 static void restore_privs(void)
 {
-    setfsuid(oldfsuid);
-    setfsgid(oldfsgid);
+    if (getuid() != 0) {
+        setfsuid(oldfsuid);
+        setfsgid(oldfsgid);
+    }
 }
 
 static int do_unmount(const char *mnt, int quiet, int lazy)
@@ -89,10 +92,10 @@ static int do_unmount(const char *mnt, int quiet, int lazy)
     return res;
 }
 
-#ifndef USE_UCLIBC
+#ifndef IGNORE_MTAB
 /* use a lock file so that multiple fusermount processes don't try and
    modify the mtab file at once! */
-static int lock_mtab()
+static int lock_mtab(void)
 {
     const char *mtab_lock = _PATH_MOUNTED ".fuselock";
     int mtablock;
@@ -117,6 +120,23 @@ static void unlock_mtab(int mtablock)
     }
 }
 
+/* Glibc addmntent() doesn't encode '\n', misencodes '\t' as '\n'
+   (version 2.3.2), and encodes '\\' differently as mount(8).  So
+   let's not allow those characters, they are not all that usual in
+   filenames. */
+static int check_name(const char *name)
+{
+    char *s;
+    for (s = "\n\t\\"; *s; s++) {
+        if (strchr(name, *s)) {
+            fprintf(stderr, "%s: illegal character 0x%02x in mount entry\n",
+                    progname, *s);
+            return -1;
+        }
+    }
+    return 0;
+}
+
 static int add_mount(const char *fsname, const char *mnt, const char *type,
                      const char *opts)
 {
@@ -125,6 +145,10 @@ static int add_mount(const char *fsname, const char *mnt, const char *type,
     struct mntent ent;
     FILE *fp;
 
+    if (check_name(fsname) == -1 || check_name(mnt) == -1 || 
+        check_name(type) == -1 || check_name(opts) == -1)
+        return -1;
+
     fp = setmntent(mtab, "a");
     if (fp == NULL) {
 	fprintf(stderr, "%s: failed to open %s: %s\n", progname, mtab,
@@ -157,6 +181,8 @@ static int remove_mount(const char *mnt, int quiet, const char *mtab,
     FILE *fp;
     FILE *newfp;
     const char *user = NULL;
+    char uidstr[32];
+    unsigned uidlen = 0;
     int found;
 
     fp = setmntent(mtab, "r");
@@ -177,22 +203,31 @@ static int remove_mount(const char *mnt, int quiet, const char *mtab,
         user = get_user_name();
         if (user == NULL)
             return -1;
+
+        uidlen = sprintf(uidstr, "%u", getuid());
     }
 
     found = 0;
     while ((entp = getmntent(fp)) != NULL) {
-        int remove = 0;
+        int removed = 0;
         if (!found && strcmp(entp->mnt_dir, mnt) == 0 &&
            strcmp(entp->mnt_type, "fuse") == 0) {
             if (user == NULL)
-                remove = 1;
+                removed = 1;
             else {
                 char *p = strstr(entp->mnt_opts, "user=");
-                if (p != NULL && strcmp(p + 5, user) == 0)
-                    remove = 1;
+                if (p && (p == entp->mnt_opts || *(p-1) == ',') &&
+                    strcmp(p + 5, user) == 0)
+                    removed = 1;
+                /* /etc/mtab is a link pointing to /proc/mounts: */
+                else if ((p = strstr(entp->mnt_opts, "user_id=")) &&
+                         (p == entp->mnt_opts || *(p-1) == ',') &&
+                         strncmp(p + 8, uidstr, uidlen) == 0 &&
+                         (*(p+8+uidlen) == ',' || *(p+8+uidlen) == '\0'))
+                    removed = 1;
             }
         }
-        if (remove)
+        if (removed)
             found = 1;
         else {
             res = addmntent(newfp, entp);
@@ -217,14 +252,14 @@ static int remove_mount(const char *mnt, int quiet, const char *mtab,
     return 0;
 }
 
-static int count_fuse_fs()
+static int count_fuse_fs(void)
 {
     struct mntent *entp;
     int count = 0;
     const char *mtab = _PATH_MOUNTED;
     FILE *fp = setmntent(mtab, "r");
     if (fp == NULL) {
-        fprintf(stderr, "%s: faild to open %s: %s\n", progname, mtab,
+        fprintf(stderr, "%s: failed to open %s: %s\n", progname, mtab,
                 strerror(errno));
         return -1;
     }
@@ -242,18 +277,12 @@ static int unmount_rename(const char *mnt, int quiet, int lazy,
     int res;
     struct stat sbuf;
 
-    if (getuid() != 0) {
-        res = drop_privs();
-        if (res == -1)
-            return -1;
-    }
+    drop_privs();
     res = do_unmount(mnt, quiet, lazy);
+    restore_privs();
     if (res == -1)
         return -1;
 
-    if (getuid() != 0)
-        restore_privs();
-
     if (stat(mtab, &sbuf) == 0)
         chown(mtab_new, sbuf.st_uid, sbuf.st_gid);
 
@@ -283,7 +312,7 @@ static int unmount_fuse(const char *mnt, int quiet, int lazy)
     }
     return 0;
 }
-#else /* USE_UCLIBC */
+#else /* IGNORE_MTAB */
 static int lock_mtab()
 {
     return 0;
@@ -313,7 +342,7 @@ static int unmount_fuse(const char *mnt, int quiet, int lazy)
 {
     return do_unmount(mnt, quiet, lazy);
 }
-#endif
+#endif /* IGNORE_MTAB */
 
 static void strip_line(char *line)
 {
@@ -427,7 +456,7 @@ static int add_option(char **optsp, const char *opt, unsigned expand)
     else {
         unsigned oldsize = strlen(*optsp);
         unsigned newsize = oldsize + 1 + strlen(opt) + expand + 1;
-        newopts = realloc(*optsp, newsize);
+        newopts = (char *) realloc(*optsp, newsize);
         if (newopts)
             sprintf(newopts + oldsize, ",%s", opt);
     }
@@ -479,9 +508,41 @@ static int opt_eq(const char *s, unsigned len, const char *opt)
         return 0;
 }
 
+static int check_mountpoint_empty(const char *mnt, mode_t rootmode,
+                                  off_t rootsize)
+{
+    int isempty = 1;
+
+    if (S_ISDIR(rootmode)) {
+        struct dirent *ent;
+        DIR *dp = opendir(mnt);
+        if (dp == NULL) {
+            fprintf(stderr, "%s: failed to mountpoint for reading: %s\n",
+                    progname, strerror(errno));
+            return -1;
+        }
+        while ((ent = readdir(dp)) != NULL) {
+            if (strcmp(ent->d_name, ".") != 0 &&
+                strcmp(ent->d_name, "..") != 0) {
+                isempty = 0;
+                break;
+            }
+        }
+        closedir(dp);
+    } else if (rootsize)
+        isempty = 0;
+
+    if (!isempty) {
+        fprintf(stderr, "%s: mountpoint is not empty\n", progname);
+        fprintf(stderr, "%s: if you are sure this is safe, use the 'nonempty' mount option\n", progname);
+        return -1;
+    }
+    return 0;
+}
+
 static int do_mount(const char *mnt, const char *type, mode_t rootmode,
                     int fd, const char *opts, const char *dev, char **fsnamep,
-                    char **mnt_optsp)
+                    char **mnt_optsp, off_t rootsize)
 {
     int res;
     int flags = MS_NOSUID | MS_NODEV;
@@ -490,8 +551,9 @@ static int do_mount(const char *mnt, const char *type, mode_t rootmode,
     const char *s;
     char *d;
     char *fsname = NULL;
+    int check_empty = 1;
 
-    optbuf = malloc(strlen(opts) + 128);
+    optbuf = (char *) malloc(strlen(opts) + 128);
     if (!optbuf) {
         fprintf(stderr, "%s: failed to allocate memory\n", progname);
         return -1;
@@ -505,14 +567,15 @@ static int do_mount(const char *mnt, const char *type, mode_t rootmode,
             unsigned fsname_str_len = strlen(fsname_str);
             if (fsname)
                 free(fsname);
-            fsname = malloc(len - fsname_str_len + 1);
+            fsname = (char *) malloc(len - fsname_str_len + 1);
             if (!fsname) {
                 fprintf(stderr, "%s: failed to allocate memory\n", progname);
-                free(optbuf);
-                return -1;
+                goto err;
             }
             memcpy(fsname, s + fsname_str_len, len - fsname_str_len);
             fsname[len - fsname_str_len] = '\0';
+        } else if (opt_eq(s, len, "nonempty")) {
+            check_empty = 0;
         } else if (!begins_with(s, "fd=") &&
                    !begins_with(s, "rootmode=") &&
                    !begins_with(s, "user_id=") &&
@@ -535,8 +598,7 @@ static int do_mount(const char *mnt, const char *type, mode_t rootmode,
                 (opt_eq(s, len, "allow_other") ||
                  opt_eq(s, len, "allow_root"))) {
                 fprintf(stderr, "%s: option %.*s only allowed if 'user_allow_other' is set in /etc/fuse.conf\n", progname, len, s);
-                free(optbuf);
-                return -1;
+                goto err;
             }
             if (!skip_option) {
                 if (find_mount_flag(s, len, &on, &flag)) {
@@ -557,22 +619,22 @@ static int do_mount(const char *mnt, const char *type, mode_t rootmode,
     }
     *d = '\0';
     res = get_mnt_opts(flags, optbuf, &mnt_opts);
-    if (res == -1) {
-        free(mnt_opts);
-        free(optbuf);
-        return -1;
-    }
+    if (res == -1)
+        goto err;
+
     sprintf(d, "fd=%i,rootmode=%o,user_id=%i,group_id=%i",
             fd, rootmode, getuid(), getgid());
     if (fsname == NULL) {
         fsname = strdup(dev);
         if (!fsname) {
             fprintf(stderr, "%s: failed to allocate memory\n", progname);
-            free(optbuf);
-            return -1;
+            goto err;
         }
     }
 
+    if (check_empty && check_mountpoint_empty(mnt, rootmode, rootsize) == -1)
+        goto err;
+
     res = mount(fsname, mnt, type, flags, optbuf);
     if (res == -1 && errno == EINVAL) {
         /* It could be an old version not supporting group_id */
@@ -581,8 +643,7 @@ static int do_mount(const char *mnt, const char *type, mode_t rootmode,
     }
     if (res == -1) {
         fprintf(stderr, "%s: mount failed: %s\n", progname, strerror(errno));
-        free(fsname);
-        free(mnt_opts);
+        goto err;
     } else {
         *fsnamep = fsname;
         *mnt_optsp = mnt_opts;
@@ -590,6 +651,12 @@ static int do_mount(const char *mnt, const char *type, mode_t rootmode,
     free(optbuf);
 
     return res;
+
+ err:
+    free(fsname);
+    free(mnt_opts);
+    free(optbuf);
+    return -1;
 }
 
 static int check_version(const char *dev)
@@ -622,11 +689,12 @@ static int check_version(const char *dev)
     return 0;
 }
 
-static int check_perm(const char **mntp, struct stat *stbuf, int *currdir_fd)
+static int check_perm(const char **mntp, struct stat *stbuf, int *currdir_fd,
+                      int *mountpoint_fd)
 {
     int res;
     const char *mnt = *mntp;
-    const char *origmnt;
+    const char *origmnt = mnt;
 
     res = lstat(mnt, stbuf);
     if (res == -1) {
@@ -639,45 +707,68 @@ static int check_perm(const char **mntp, struct stat *stbuf, int *currdir_fd)
     if (getuid() == 0)
         return 0;
 
-    if (!S_ISDIR(stbuf->st_mode)) {
-        fprintf(stderr, "%s: mountpoint %s is not a directory\n",
+    if (S_ISDIR(stbuf->st_mode)) {
+        *currdir_fd = open(".", O_RDONLY);
+        if (*currdir_fd == -1) {
+            fprintf(stderr, "%s: failed to open current directory: %s\n",
+                    progname, strerror(errno));
+            return -1;
+        }
+        res = chdir(mnt);
+        if (res == -1) {
+            fprintf(stderr, "%s: failed to chdir to mountpoint: %s\n",
+                    progname, strerror(errno));
+            return -1;
+        }
+        mnt = *mntp = ".";
+        res = lstat(mnt, stbuf);
+        if (res == -1) {
+            fprintf(stderr, "%s: failed to access mountpoint %s: %s\n",
+                    progname, origmnt, strerror(errno));
+            return -1;
+        }
+
+        if ((stbuf->st_mode & S_ISVTX) && stbuf->st_uid != getuid()) {
+            fprintf(stderr, "%s: mountpoint %s not owned by user\n",
+                    progname, origmnt);
+            return -1;
+        }
+
+        res = access(mnt, W_OK);
+        if (res == -1) {
+            fprintf(stderr, "%s: user has no write access to mountpoint %s\n",
+                    progname, origmnt);
+            return -1;
+        }
+    } else if (S_ISREG(stbuf->st_mode)) {
+        static char procfile[256];
+        *mountpoint_fd = open(mnt, O_WRONLY);
+        if (*mountpoint_fd == -1) {
+            fprintf(stderr, "%s: failed to open %s: %s\n", progname, mnt,
+                    strerror(errno));
+            return -1;
+        }
+        res = fstat(*mountpoint_fd, stbuf);
+        if (res == -1) {
+            fprintf(stderr, "%s: failed to access mountpoint %s: %s\n",
+                    progname, mnt, strerror(errno));
+            return -1;
+        }
+        if (!S_ISREG(stbuf->st_mode)) {
+            fprintf(stderr, "%s: mountpoint %s is no longer a regular file\n",
+                    progname, mnt);
+            return -1;
+        }
+
+        sprintf(procfile, "/proc/self/fd/%i", *mountpoint_fd);
+        *mntp = procfile;
+    } else {
+        fprintf(stderr,
+                "%s: mountpoint %s is not a directory or a regular file\n",
                 progname, mnt);
         return -1;
     }
 
-    *currdir_fd = open(".", O_RDONLY);
-    if (*currdir_fd == -1) {
-        fprintf(stderr, "%s: failed to open current directory: %s\n",
-                progname, strerror(errno));
-        return -1;
-    }
-    res = chdir(mnt);
-    if (res == -1) {
-        fprintf(stderr, "%s: failed to chdir to mountpoint: %s\n",
-                progname, strerror(errno));
-        return -1;
-    }
-    origmnt = mnt;
-    mnt = *mntp = ".";
-    res = lstat(mnt, stbuf);
-    if (res == -1) {
-        fprintf(stderr, "%s: failed to access mountpoint %s: %s\n",
-                progname, origmnt, strerror(errno));
-        return -1;
-    }
-
-    if ((stbuf->st_mode & S_ISVTX) && stbuf->st_uid != getuid()) {
-        fprintf(stderr, "%s: mountpoint %s not owned by user\n",
-                progname, origmnt);
-        return -1;
-    }
-
-    res = access(mnt, W_OK);
-    if (res == -1) {
-        fprintf(stderr, "%s: user has no write access to mountpoint %s\n",
-                progname, origmnt);
-        return -1;
-    }
 
     return 0;
 }
@@ -701,89 +792,35 @@ static int try_open(const char *dev, char **devp, int silent)
     return fd;
 }
 
-#define FUSE_TMP_DIRNAME "/tmp/.fuse_devXXXXXX"
-#define FUSE_TMP_DEVNAME "/fuse"
-
-static int try_open_new_temp(dev_t devnum, char **devp)
-{
-    int res;
-    int fd;
-    char dirname[] = FUSE_TMP_DIRNAME;
-    char filename[] = FUSE_TMP_DIRNAME FUSE_TMP_DEVNAME;
-    if (mkdtemp(dirname) == NULL) {
-        fprintf(stderr, "%s: failed to create temporary device directory: %s\n",
-                progname, strerror(errno));
-        return -1;
-    }
-    sprintf(filename, "%s%s", dirname, FUSE_TMP_DEVNAME);
-    res = mknod(filename, S_IFCHR | 0600, devnum);
-    if (res == -1) {
-        fprintf(stderr, "%s: failed to create device node: %s\n", progname,
-                strerror(errno));
-        rmdir(dirname);
-        return -1;
-    }
-    fd = try_open(filename, devp, 0);
-    unlink(filename);
-    rmdir(dirname);
-    return fd;
-}
-
 static int try_open_fuse_device(char **devp)
 {
-    int fd = try_open(FUSE_DEV_NEW, devp, 1);
+    int fd;
+    int err;
+
+    drop_privs();
+    fd = try_open(FUSE_DEV_NEW, devp, 0);
+    restore_privs();
     if (fd >= 0)
         return fd;
 
-    if (fd == -1) {
-        fd = try_open_new_temp(makedev(FUSE_MAJOR, FUSE_MINOR), devp);
-        if (fd != -2)
-            return fd;
-    }
-
+    err = fd;
     fd = try_open(FUSE_DEV_OLD, devp, 1);
     if (fd >= 0)
         return fd;
 
-    return -1;
+    return err;
 }
 
 static int open_fuse_device(char **devp)
 {
-    int fd;
-
-    if (1
-#ifndef AUTO_MODPROBE
-        && getuid() == 0
-#endif
-        ) {
-        int status;
-        pid_t pid;
-
-        fd = try_open_fuse_device(devp);
-        if (fd >= 0)
-            return fd;
-
-#ifndef USE_UCLIBC
-        pid = fork();
-#else
-        pid = vfork();
-#endif
-        if (pid == 0) {
-            setuid(0);
-            execl("/sbin/modprobe", "/sbin/modprobe", "fuse", NULL);
-            exit(1);
-        }
-        if (pid != -1)
-            waitpid(pid, &status, 0);
-    }
-
-    fd = try_open_fuse_device(devp);
+    int fd = try_open_fuse_device(devp);
     if (fd >= 0)
         return fd;
 
-    fprintf(stderr, "%s: fuse device not found, try 'modprobe fuse' first\n",
-            progname);
+    if (fd == -2)
+        fprintf(stderr,
+                "%s: fuse device not found, try 'modprobe fuse' first\n",
+                progname);
     return -1;
 }
 
@@ -795,10 +832,11 @@ static int mount_fuse(const char *mnt, const char *opts)
     char *dev;
     const char *type = "fuse";
     struct stat stbuf;
-    char *fsname;
-    char *mnt_opts;
+    char *fsname = NULL;
+    char *mnt_opts = NULL;
     const char *real_mnt = mnt;
     int currdir_fd = -1;
+    int mountpoint_fd = -1;
     int mtablock = -1;
 
     fd = open_fuse_device(&dev);
@@ -813,15 +851,7 @@ static int mount_fuse(const char *mnt, const char *opts)
         }
     }
 
-    if (getuid() != 0) {
-        res = drop_privs();
-        if (res == -1) {
-            close(fd);
-            unlock_mtab(mtablock);
-            return -1;
-        }
-    }
-
+    drop_privs();
     read_conf();
 
     if (getuid() != 0 && mount_max != -1) {
@@ -836,26 +866,27 @@ static int mount_fuse(const char *mnt, const char *opts)
 
     res = check_version(dev);
     if (res != -1) {
-        res = check_perm(&real_mnt, &stbuf, &currdir_fd);
+        res = check_perm(&real_mnt, &stbuf, &currdir_fd, &mountpoint_fd);
+        restore_privs();
         if (res != -1)
             res = do_mount(real_mnt, type, stbuf.st_mode & S_IFMT, fd, opts,
-                           dev, &fsname, &mnt_opts);
-    }
-
-    if (getuid() != 0)
+                           dev, &fsname, &mnt_opts, stbuf.st_size);
+    } else
         restore_privs();
 
+    if (currdir_fd != -1) {
+        fchdir(currdir_fd);
+        close(currdir_fd);
+    }
+    if (mountpoint_fd != -1)
+        close(mountpoint_fd);
+
     if (res == -1) {
         close(fd);
         unlock_mtab(mtablock);
         return -1;
     }
 
-    if (currdir_fd != -1) {
-        fchdir(currdir_fd);
-        close(currdir_fd);
-    }
-
     if (geteuid() == 0) {
         res = add_mount(fsname, mnt, type, mnt_opts);
         unlock_mtab(mtablock);
@@ -882,6 +913,11 @@ static char *resolve_path(const char *orig)
     char *lastcomp;
     const char *toresolv;
 
+    if (!orig[0]) {
+        fprintf(stderr, "%s: invalid mountpoint '%s'\n", progname, orig);
+        return NULL;
+    }
+
     copy = strdup(orig);
     if (copy == NULL) {
         fprintf(stderr, "%s: failed to allocate memory\n", progname);
@@ -970,12 +1006,13 @@ static int send_fd(int sock_fd, int fd)
     return 0;
 }
 
-static void usage()
+static void usage(void)
 {
     fprintf(stderr,
             "%s: [options] mountpoint\n"
             "Options:\n"
             " -h                print help\n"
+            " -v                print version\n"
             " -o opt[,opt...]   mount options\n"
             " -u                unmount\n"
             " -q                quiet\n"
@@ -984,38 +1021,52 @@ static void usage()
     exit(1);
 }
 
+static void show_version(void)
+{
+    printf("%s\n", PACKAGE_STRING);
+    exit(0);
+}
+
 int main(int argc, char *argv[])
 {
-    int a;
+    int ch;
     int fd;
     int res;
     char *origmnt;
     char *mnt;
-    int unmount = 0;
-    int lazy = 0;
+    static int unmount = 0;
+    static int lazy = 0;
+    static int quiet = 0;
     char *commfd;
-    int quiet = 0;
     int cfd;
     const char *opts = "";
 
-    progname = argv[0];
+    static const struct option long_opts[] = {
+        {"unmount", no_argument, NULL, 'u'},
+        {"lazy",    no_argument, NULL, 'z'},
+        {"quiet",   no_argument, NULL, 'q'},
+        {"help",    no_argument, NULL, 'h'},
+        {"version", no_argument, NULL, 'v'},
+        {0, 0, 0, 0}};
 
-    for (a = 1; a < argc; a++) {
-        if (argv[a][0] != '-')
-            break;
+    progname = strdup(argv[0]);
+    if (progname == NULL) {
+        fprintf(stderr, "%s: failed to allocate memory\n", argv[0]);
+        exit(1);
+    }
 
-        switch (argv[a][1]) {
+    while ((ch = getopt_long(argc, argv, "hvo:uzq", long_opts, NULL)) != -1) {
+        switch (ch) {
         case 'h':
             usage();
             break;
 
+        case 'v':
+            show_version();
+            break;
+
         case 'o':
-            a++;
-            if (a == argc) {
-                fprintf(stderr, "%s: missing argument to -o\n", progname);
-                exit(1);
-            }
-            opts = argv[a];
+            opts = optarg;
             break;
 
         case 'u':
@@ -1031,32 +1082,28 @@ int main(int argc, char *argv[])
             break;
 
         default:
-            fprintf(stderr, "%s: unknown option %s\n", progname, argv[a]);
-            fprintf(stderr, "Try `%s -h' for more information\n", progname);
             exit(1);
         }
     }
 
-    if (a == argc) {
+    if (lazy && !unmount) {
+        fprintf(stderr, "%s: -z can only be used with -u\n", progname);
+        exit(1);
+    }
+
+    if (optind >= argc) {
         fprintf(stderr, "%s: missing mountpoint argument\n", progname);
         exit(1);
     }
 
-    origmnt = argv[a++];
-
-    if (getuid() != 0) {
-        res = drop_privs();
-        if (res == -1)
-            exit(1);
-    }
+    origmnt = argv[optind];
 
+    drop_privs();
     mnt = resolve_path(origmnt);
+    restore_privs();
     if (mnt == NULL)
         exit(1);
 
-    if (getuid() != 0)
-        restore_privs();
-
     umask(033);
     if (unmount) {
         if (geteuid() == 0) {
